
//===========================================================================//
//                                                                           //
//  Copyright (C) 2004 - 2018                                                //
//  IDS Imaging GmbH                                                         //
//  Dimbacherstr. 6-8                                                        //
//  D-74182 Obersulm-Willsbach                                               //
//                                                                           //
//  The information in this document is subject to change without            //
//  notice and should not be construed as a commitment by IDS Imaging GmbH.  //
//  IDS Imaging GmbH does not assume any responsibility for any errors       //
//  that may appear in this document.                                        //
//                                                                           //
//  This document, or source code, is provided solely as an example          //
//  of how to utilize IDS software libraries in a sample application.        //
//  IDS Imaging GmbH does not assume any responsibility for the use or       //
//  reliability of any portion of this document or the described software.   //
//                                                                           //
//  General permission to copy or modify, but not for profit, is hereby      //
//  granted,  provided that the above copyright notice is included and       //
//  reference made to the fact that reproduction privileges were granted	 //
//	by IDS Imaging GmbH.				                                     //
//                                                                           //
//  IDS cannot assume any responsibility for the use or misuse of any        //
//  portion of this software for other than its intended diagnostic purpose	 //
//  in calibrating and testing IDS manufactured cameras and software.		 //
//                                                                           //
//===========================================================================//


// uEyeDigitalCropDlg.h : header file
//

#pragma once

#include <vector>
#include "uEye.h"
#include "afxwin.h"
#include "afxcmn.h"


class CuEyeDigitalCropDlgAutoProxy;

// CuEyeDigitalCropDlg dialog
class CuEyeDigitalCropDlg : public CDialogEx
{
// Construction
public:
	CuEyeDigitalCropDlg(CWnd* pParent = NULL);	// standard constructor


// some defines for the GUI
#define CROP_MODE_OFF				0x00
#define CROP_MODE_SUBSAMPLING		0x01
#define CROP_MODE_BINNING			0x02
#define CROP_BINNING_AVERAGE				0x10
#define CROP_BINNING_ADDITIVE				0x20
#define CROP_BINNING_H_ADDITIVE_V_AVERAGE	0x40
#define CROP_BAYER_SCALING			0x100
#define CROP_MIN	2
#define CROP_MAX	16
#define CROP_STEP	2


	// camera variables
	HIDS	m_hCam;					// handle to camera
	HWND	m_hWndDisplayOrg;		// handle to display window of original camera image
	INT		m_nColorMode;			// e.g. MONO8
	INT		m_nBitsPerPixel;		// number of bits needed to store one pixel
	INT		m_nSizeX;				// width of video 
	INT		m_nSizeY;				// height of video
	INT		m_nPitch;				// line pitch, numbers in bytes per line
	INT		m_dwPitch;				// line pitch, numbers in DWORD (4bytes) per line
	SENSORINFO m_SensorInfo;		// sensor information struct
	double	m_fps;					// camera fps
	bool	m_bLive;				// camera is in live mode (true)
	bool	m_bColor;				// we have a color camera (true)
	bool	m_bDisplayOrgImage;		// turn display for original image on/off (true/false)
	bool	m_bDisplayCropImage;	// turn display for crop image on/off (true/false)


	// memory and sequence buffers for the image queue
	std::vector<INT> m_viSeqMemId;		// camera memory - buffer IDs
	std::vector<char*> m_vpcSeqImgMem;	// camera memory - pointers to buffer
	int		m_nNumberOfBuffers;			// the number of image buffers we use
	
	// crop variables
	HWND	m_hWndDisplayCrop;		// handle to display window of cropped image
	INT		m_nCropSizeX;			// width of cropped video 
	INT		m_nCropAdjustX;			// only for mono cameras and odd scaling factors required
	INT		m_nCropSizeY;			// height of cropped video
	INT		m_nCropLinePitch;		// line pitch, numbers in bytes per line
	INT		m_dwCropLinePitch;		// line pitch, numbers in DWORD (4bytes) per line
	INT		m_nCropColorMode;		// e.g. MONO8
	INT		m_nCropBitsPerPixel;	// number of bits needed to store one pixel
	INT		m_nCropMemId;			// crop memory - buffer ID
	char*	m_pcCropImgMem;			// crop memory - pointer to buffer
	INT		m_nCropMode;			// which crop mode do we run
	int		m_nSubSamplingFactor;	// current factor for sub sampling
	int		m_nBinningFactor;		// current factor for binning
	int		m_nBinningMode;			// average or mean mode for binning (see defines)


	// thread variables
	HANDLE  m_hThreadImageQueue;
	BOOL    m_boRunThreadImageQueue;
	BOOL    m_boTerminatedThreadImageQueue;
	DWORD   m_dwThreadIDImageQueue;


	// camera functions
	INT InitCamera(HIDS *hCam, HWND hWnd);
	bool CamOpen();
	bool CamRunImageQueue(void);
	bool CamTerminateImageQueue(void);
	bool CamSeqBuild(INT ColorMode);
	bool CamSeqKill();
	bool CamToggleLive();


	// crop functions
	bool CropPrepareSubSampling(void);
	bool CropCleanupSubSampling(void);
	bool CropDoSubSampling(int nFactor, char* pcSourceImgMem);
	void DoSubSamplingMono(int nFactor, char* pcSourceImgMem);
	void DoSubSamplingColor(int nFactor, char* pcSourceImgMem);
	bool CropPrepareBinning(void);
	bool CropCleanupBinning(void);
	bool CropDoBinning(int nFactor, char* pcSourceImgMem);
	void DoBinningMonoAverage(int nFactor, char* pcSourceImgMem);
	void DoBinningMonoAdditive(int nFactor, char* pcSourceImgMem);
	void DoBinningMonoAdditiveAverage(int nFactor, char* pcSourceImgMem);
	void DoBinningColorAverage(int nFactor, char* pcSourceImgMem);
	void DoBinningColorAdditive(int nFactor, char* pcSourceImgMem);
	void DoBinningColorAdditiveAverage(int nFactor, char* pcSourceImgMem);
	bool CropPrepareBayerScaling(void);
	bool CropCleanupBayerScaling(void);
	bool CropDoBayerScaling(char* pcSourceImgMem);
	void GuiCropClean(void);

	
	// thread function
	void ThreadProcImageQueue();	// thread for collecting the image data



// Dialog Data
#ifdef AFX_DESIGN_TIME
	enum { IDD = IDD_UEYEDIGITALCROP_DIALOG };
#endif

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support


// Implementation
protected:
	CuEyeDigitalCropDlg* m_pAutoProxy;
	HICON m_hIcon;

	BOOL CanExit();

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnClose();
	virtual void OnOK();
	virtual void OnCancel();
	DECLARE_MESSAGE_MAP()
public:

	afx_msg void OnBnClickedButtonInit();
	afx_msg void OnBnClickedButtonExit();
	afx_msg void OnBnClickedButtonSnap();
	afx_msg void OnBnClickedButtonLive();
	afx_msg void OnBnClickedRadioCropOff();
	afx_msg void OnBnClickedRadioCropBinning();
	afx_msg void OnBnClickedRadioCropSubsampling();
	afx_msg void OnBnClickedRadioBinningAverage();
	afx_msg void OnBnClickedRadioBinningAdditive();
	afx_msg void OnBnClickedRadioBinningAdditiveAverage();
	afx_msg void OnBnClickedRadioCropBayerBinning();
	afx_msg void OnBnClickedCheckDisplayOrg();	
	afx_msg void OnNMCustomdrawSliderFactor(NMHDR *pNMHDR, LRESULT *pResult);
	CString m_strCamType;
	CString m_strCamFps;
	CString m_strWidth;
	CString m_strHeight;
	CString m_strCropWidth;
	CString m_strCropHeight;
	CString m_strStepMin;
	CString m_strStepCurrent;
	CString m_strStepMax;
	CSliderCtrl m_ctrlSliderFactor;
	CButton m_ctrlBtnInit;
	CButton m_ctrlBtnSnap;
	CButton m_ctrlBtnLive;
	CButton m_ctrlRadioBtnCropOff;
	CButton m_ctrlRadioBtnCropSubSampling;
	CButton m_ctrlRadioBtnCropBinning;
	CButton m_ctrlRadioBtnBinningAverage;
	CButton m_ctrlRadioBtnBinningAdditive;
	CButton m_ctrlRadioBtnBinningAdditiveAverage;
	CButton m_ctrlRadioBtnCropBayerScaling;
	CButton m_ctrlCheckBtnDisplayOrgImage;	
	CButton m_ctrlCheckBtnDisplayCropImage;
	CStatic m_ctrlStaticDisplayOrg;
	afx_msg void OnBnClickedCheckDisplayCrop();
};
